﻿namespace ControllerPropertyInjectTestWebApp
{
    using System.Linq;

    using Microsoft.AspNetCore.Mvc.Controllers;
    using Microsoft.Extensions.DependencyInjection;
    using Microsoft.Extensions.DependencyInjection.Extensions;

    public static class AutowiredControllerServiceProviderExtensions
    {
        public static void AutowiredControllerServiceProvider(this IMvcBuilder mvcBuilder)
        {
            var services = mvcBuilder.Services;

            //查找原始服务描述
            var serviceDescriptor = services.Where(m => m.ServiceType == typeof(IControllerActivator)).First();

            var existedServiceType = serviceDescriptor.ImplementationType;

            //动态创建代理类型
            var controllerActivatorType = typeof(ControllerActivatorProxy<>).MakeGenericType(existedServiceType);

            //将原始实现直接添加到services中
            services.Add(ServiceDescriptor.Describe(existedServiceType, existedServiceType, serviceDescriptor.Lifetime));

            //替换IControllerActivator服务为使用动态构建的代理类型
            services.Replace(ServiceDescriptor.Describe(typeof(IControllerActivator), controllerActivatorType, serviceDescriptor.Lifetime));
        }
    }
}
