﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;


namespace ControllerPropertyInjectTestWebApp
{
    using Microsoft.AspNetCore.Mvc;
    using Microsoft.AspNetCore.Mvc.Controllers;

    public class ControllerActivatorProxy<TControllerActivator> : IControllerActivator where TControllerActivator : IControllerActivator
    {
        private readonly TControllerActivator _originalControllerActivator;

        public ControllerActivatorProxy(TControllerActivator originalControllerActivator)
        {
            //引用原始ControllerActivator
            _originalControllerActivator = originalControllerActivator;
        }

        public object Create(ControllerContext context)
        {
            //使用原始ControllerActivator创建controller
            var controller = _originalControllerActivator.Create(context);

            if (controller is IServiceProviderSetter serviceProviderSetter)
            {
                //具体要干什么由controller内部决定
                serviceProviderSetter.SetServiceProvider(context.HttpContext.RequestServices);
            }

            return controller;
        }

        public void Release(ControllerContext context, object controller)
        {
            _originalControllerActivator.Release(context, controller);
        }
    }
}
